//
//
//

#include "h.h"
#ifdef PIC18
#include "p18f452.h"
#endif
#ifdef MSDOS
#include "stdio.h"
#include "conio.h"
#endif

//DEFINICIONES
#ifdef MSDOS
u8 CCPR1L,CCPR2L,TRISD,PORTLCD;
char eeprom[0x0100];
#endif

#ifdef PIC18
#define PORTLCD PORTD
#endif
#define E1 0x08
#define RW 0x40
#define RS 0x02
#define nop

#define enable()  INTCONbits.GIE=1
#define disable() INTCONbits.GIE=0

//PROTOTIPOS
void PROCESOInterrupcion(void);  //en proceso.c
void EEPROMWrite(u16,u8);
#ifdef PIC18
void putchar(u8);
#endif

//VARIABLES
//privado
#pragma udata u4
char lcdram[TAMVTAMH];// RAM pantalla
#pragma udata u3
u16  p;               // cursor
bit  bPaint;          // actualizar la pantalla
//public
u32 temps;            // tiempo del sistema (en msegundos), se desborda 49d
u8  tempsu;           // tiempo del sistema (en centesimas de milisegundo)
u8  _PORTA,_PORTB,_PORTE;
//#define TEMPSU 19531// 20000000/1024=19531.25
#define TEMPSU 2000   // 20000000/4/250=20000

//PROGRAMA
#pragma code general

void gotox1y1(void)
{
#ifndef PIC18
gotoxy(1,1);
_setcursortype(_NOCURSOR);
//printf(" .123456789.123456789.123456789.123456789\n");
#endif
#ifdef PIC18
putchar(0x1b);
putchar('[');
putchar('H');
#endif
}


void LCDDisplay(void)
{
u16 i,j;
if(bPaint==0)return;
bPaint=0;
gotox1y1();
for(i=0;i!=TAMV;i++)
  {
  putchar('*');
  for(j=0;j!=TAMH;j++)putchar(lcdram[(i*TAMH)+j]);
  putchar('*');
  putchar('\n');
  putchar('\r');
  }
}


void LCDReset(void)
{
u16 n;
p=0;
for(n=0;n!=TAMVTAMH;n++)lcdram[n]=' ';
bPaint=1;
#ifndef PIC18
printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
#endif
}

void LCDGotoXY(u8 x,u8 y)
{
p=y*TAMH+x;
}

void LCDWrite(char c)
{
if(p<TAMVTAMH)lcdram[p++]=c;
bPaint=1;
}


u8 KEYBOARDRead(void)
{
char c;
#ifndef PIC18
if(kbhit())return getch();
#endif
#ifdef PIC18
if(PIR1bits.RCIF)
	{
	c=RCREG;
    PIR1bits.RCIF=0;
	if(RCSTAbits.OERR)
		{
		RCSTAbits.CREN=0;
		RCSTAbits.CREN=1;
		}
	return c;
	}
#endif
return 0;
}



void EEPROMWrite(u16 a,u8 d)
{
#ifdef MSDOS
//printf("write eeprom[%d]=%d\n",a,d);
FILE *out;
eeprom[a]=d;
out=fopen("eeprom.bin","wb");
if(out)fwrite(eeprom,1,0x0100,out);
fclose(out);
#endif
#ifdef PIC18
EEADR=(u8)a;
EEDATA=d;
EECON1bits.EEPGD=0;
EECON1bits.CFGS=0;
EECON1bits.WREN=1;
disable();
EECON2=0x55;
EECON2=0xAA;
EECON1bits.WR=1;
enable();
while(EECON1bits.WR);  //espero finalizacion grabacion!
EECON1bits.WREN=0;
#endif
}

u8 EEPROMRead(u16 a)
{
u8 x;
x=0xaa;
#ifdef PIC18
EEADR=(u8)a;
EECON1bits.EEPGD=0;   //EEPROM=0 FLASH=1
EECON1bits.CFGS=0;    //FLASH/EEPROM=0  CFG-BITS=1
EECON1bits.RD=1;
x=EEDATA;
//while(EECON1bits.RD); //espero finalizacion lectura!
#endif
#ifdef MSDOS
x=eeprom[a];
//printf("read %d=eeprom[%d]\n",x,a);
#endif
return x;
}


bit bEEPROM;
void EEPROMReset(void)
{
u8 id;
#ifdef MSDOS
FILE *in;
in=fopen("eeprom.bin","rb");
fread(eeprom,1,0x0100,in);
fclose(in);
#endif
id=EEPROMRead(0);
if(id==0xaa)bEEPROM=1;
else        bEEPROM=0;
}

/////////////////////////////////////////////////////////////////

void INTReset(void)
{
#ifdef PIC18
PIR1bits.TMR2IF=0;
PIE1bits.TMR2IE=1;
INTCONbits.PEIE=1;
INTCONbits.GIE =1;
#endif
}


#ifdef PIC18
void putchar(char c)
{
u8 n;
while(PIR1bits.TXIF==0);
for(n=0xff;n;n--)ClrWdt();
TXREG=c;
}
#endif

void RS232Reset(void)
{
#ifdef PIC18
PIR1bits.RCIF=0;
PIE1bits.RCIE=0;
//PIR1bits.TXIF=0; se pone a 1 cuando TXREG vacio, o a 0 si lleno
PIE1bits.TXIE=0;
SPBRG=129;       //9600
RCSTA=0x90;      //serial enable,recepcion continua
TXSTA=0x24;      //tx enable, high speed
#endif
//putchar('H');
//putchar('O');
//putchar('L');
//putchar('A');
}

void PWMReset(void)
{
#ifdef PIC18
TRISCbits.TRISC2=0; //bit salida PWM1
TRISCbits.TRISC1=0; //bit salida PWM2
CCP1CON=0;
CCP2CON=0;
TMR2=0;
//PR2=0xFF; pwm 19khz
PR2=250;  // pwm 20khz, 20.000 int/s
CCPR1L=0X1F;
CCPR2L=0X7F;
CCP1CON=0x3c;
CCP2CON=0x3c;
PORTCbits.RC2=1; //bit salida PWM1
PORTCbits.RC1=1; //bit salida PWM2
#endif
}

void PWMEnable(void)
{
#ifdef PIC18
T2CON=0X04;
enable();
#endif
}

u16 wdac1,wdac2;
void PWMWrite(void)
{
CCPR1L=(u8)wdac1;
CCPR2L=(u8)wdac2;
}

void IOReset(void)
{
#ifdef PIC18
TRISAbits.TRISA0=1; // RA0 entrada analogica
TRISAbits.TRISA1=1; // RA1 entrada analogica
TRISAbits.TRISA2=1; // RA2 entrada digital
TRISAbits.TRISA3=1; // RA3 entrada analogica

PORTAbits.RA4=1;    // rele off
PORTAbits.RA5=1;    // rele off
TRISAbits.TRISA4=0; // RA4 rele IC
TRISAbits.TRISA5=0; // RA5 rele IC
//
TRISBbits.TRISB1=1; // RB1 AFN
TRISBbits.TRISB2=1; // RB2 DES

TRISEbits.TRISE0=0; // elevador tension pwm
PORTEbits.RE0=0;    // transistor en corte
TRISEbits.TRISE1=0; // led
TRISEbits.TRISE2=0; // led
PORTEbits.RE1=1;    // led apagado
PORTEbits.RE2=1;    // led apagado

TRISD=0;            // direccionamiento LCD
#endif
}

//MEDIR LA PENALIZACION EN 100% DE SEGUNDO
//20000 veces por segundo
//apilo algo?
//pongo un bit a 1
//pongo un bit a 1
//compruebo que un registro de 8 bits no es cero
//desapilo algo?
//retorno
#pragma interrupt interrupcion
void interrupcion(void)
{
#ifdef PIC18
PIR1bits.TMR2IF=0;
ClrWdt();	// reinicio watchdog
#endif
if(!tempsu)
	{
	temps++;
	tempsu=TEMPSU+1;
    PROCESOInterrupcion();
	}
tempsu--;
}

/////////////////////////////////////////////////////////////////

void exitbootloader(void)
{
EEPROMWrite(0xffff,0xff);
#ifndef PIC18
exit(1);
#endif
#ifdef PIC18;
disable();
while(1);
#endif
}

void activatebootloader(void)
{
EEPROMWrite(0xff  ,0xff);
EEPROMWrite(0xffff,0xff);
}


void inicializa(void)
{
EEPROMWrite(0,0);
#ifdef PIC18
disable();
#endif
while(1);
}



void HWReset(void)
{
//VARIABLES DEL SISTEMA
temps=1;
tempsu=TEMPSU;
}


void HWLoop(void)
{
#ifndef PIC18
static long n=80000;if(--n==0){n=80000;temps++;}
interrupcion();
#endif
}

void HWStop(void)
{
}




/*
LCD PIC
D7  RD7
D6  RD6
D5  RD5
D4  RD4
E   RD3
RW  RD2  siempre a 0, anular en un futuro
RS  RD1
	RD0
*/

u8 a,b;

void Espera15ms(void)
{
u16 n;
//for(n=0x7fff;n;n--);
}

void LCD44780BusWrite(void)
{
u8 n;
PORTLCD=a|E1; nop; PORTLCD=a&~E1; nop;nop;
PORTLCD=b|E1; nop; PORTLCD=b&~E1; nop;nop;
for(n=250;
	n;
	n--); //50uS
}

void LCD44780GoToX(u8 pos)
{
pos|=0x80;
a= pos    &0xf0;
b=(pos<<4)&0xf0;
LCD44780BusWrite();
}

void LCD44780Write(u8 l)
{
a= l    &0xf0;
b=(l<<4)&0xf0;
a|=RS; //dato=1
b|=RS; //dato=1
LCD44780BusWrite();
}

void LCD44780Reset(void)
{
TRISD=0x00;  //todo pins de salida
Espera15ms();
a=0x30;
b=a;
LCD44780BusWrite();
Espera15ms();
a=0x30;
b=a;
LCD44780BusWrite();
Espera15ms();
a=0x30;
b=a;
LCD44780BusWrite();
a=0x20;
b=a;
LCD44780BusWrite();  //bus a 4 bits
a=0x20;
b=0xf0;
LCD44780BusWrite();
a=0x00; b=0x80; LCD44780BusWrite();  //display off
a=0x00; b=0xf0; LCD44780BusWrite();  //display,cursor,parpadeo on
a=0x00; b=0x60; LCD44780BusWrite();  //autoincremento, sin desplazamiento
//a=0x00; b=0xC0; LCD44780BusWrite();
}


u8 nadc;
u16 wadc0,wadc1,wadc2,wadc3,wtiempoespera;
void ADCReset(void)
{
#ifdef PIC18
ADCON0=0x81; // ADC on, fclkadc=fosc/32,AN0
ADCON1=0xc4; // AN0 AN1 AN3, justificado a la IZQ
PIR1bits.ADIF=0;
ADCON0bits.GO=1;
#endif
nadc=0;
wtiempoespera=0x0fff;
}


//muestreo asincrono
void ADCRead(void)
{
#ifdef PIC18
u16 w;
u8 h,l;
                      //hasta aqui, he hecho track de vin
ADCON0bits.GO=1;      //inicio la conversion !!
while(ADCON0bits.GO); //llamada bloqueante
h=ADRESH;
l=ADRESL;
w=h;
w<<=8;
w&=0xff00;
w|=l;
switch(nadc)
    {
    case 0:
        wadc0=w;     
        ADCON0=0x89; //ADC1 10(00 1)001 an1 ra1
        break;
    case 1:
        wadc1=w;
        ADCON0=0x91; //ADC2 10(01 0)001 an2 ra2    (no disponible)
        break;
    case 2:
        wadc2=w;
        ADCON0=0x99; //ADC2 10(01 1)001 an3 ra3
        break;
    case 3:
        wadc3=w;
        ADCON0=0x81; //ADC3 10(00 0)001  an0 ra0
        break;
    }
nadc++;
nadc&=0x03;
PIR1bits.ADIF=0;
#endif

#ifndef PIC18
wadc0++;
wadc0&=0x03ff;
//wadc1++;
//wadc1&=0x03ff;
wadc2++;
wadc2&=0x03ff;
wadc3++;
wadc3&=0x03ff;
#endif
}

void lee_port(void)
{
#ifdef PIC18
_PORTA=PORTA;
_PORTB=PORTB;
_PORTE=PORTE;
#endif
}

